package com.ruoyi.ems.mapper;

import com.ruoyi.ems.domain.AreaAttr;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Set;

/**
 * 区域对象属性Mapper接口
 *
 * @author ruoyi
 * @date 2024-11-01
 */
public interface AreaAttrMapper {
    /**
     * 查询区域对象属性
     *
     * @param areaCode 区域对象代码
     * @return 区域对象属性
     */
    AreaAttr selectAreaAttrByCode(String areaCode);

    /**
     * 查询区域对象属性列表
     *
     * @param list 区域对象代码集合
     * @return 区域对象属性集合
     */
    List<AreaAttr> selectAreaAttrList(@Param("set") Set<String> set);

    /**
     * 新增区域对象属性
     *
     * @param areaAttr 区域对象属性
     * @return 结果
     */
    int insertAreaAttr(AreaAttr areaAttr);

    /**
     * 修改区域对象属性
     *
     * @param areaAttr 区域对象属性
     * @return 结果
     */
    int updateAreaAttr(AreaAttr areaAttr);

    /**
     * 删除区域对象属性
     *
     * @param areaCode 区域对象代码
     * @return 结果
     */
    int deleteAreaAttrByCode(String areaCode);
}
